#include <stdio.h>
#include <stdlib.h>
#include <math.h>

struct hack {
	int aspect_width;
	int aspect_height;
	int scrnwidth;
	int scrnheight;
	int scaling;
} x_cvstat = {4, 3, 640, 400, 1};

/*
 * Given a width/height of a source image, adjust it to match the current aspect
 * ratio.  Will not reduce either number
 */
static void
aspect_correct(int *x, int *y)
{
        int width = *x;
        int height;

        if (!x_cvstat.aspect_height || !x_cvstat.aspect_width)
                return;
        height = lround((double)(width * x_cvstat.aspect_height) / x_cvstat.aspect_width);
        if (height < *y) {
                height = *y * x_cvstat.scaling;
                width = lround(((double)height * x_cvstat.aspect_width) / x_cvstat.aspect_height);
        }
         
        *x = width;
        *y = height;
}

/*
 * Essentially the opposite of the above.  Given an output width/height, translates to
 * the size of the source image.
 *
 * Note that this is much trickier as the "source" bitmap may have been integer scaled
 * differently in both directions... so what this does is reverse the aspect ratio
 * calculation, then find the next lowest even multiple of the mode bitmap size.
 */
static void
aspect_reverse(int *x, int *y)
{
        int width = *x;
        int height = *y;

        if (!x_cvstat.aspect_height || !x_cvstat.aspect_width) {
                width = x_cvstat.scrnwidth * (*x / x_cvstat.scrnwidth);
                if (width < x_cvstat.scrnwidth)
                        width = x_cvstat.scrnwidth;
                height = x_cvstat.scrnheight * (*x / x_cvstat.scrnheight);
                if (height < x_cvstat.scrnheight)
                        height = x_cvstat.scrnheight;
                return;
        }
        // First, find the "controlling" dimension... the one that won't be scaled.
        if (lround((double)(width * x_cvstat.aspect_height) / x_cvstat.aspect_width) < height) {
                // Width controls
                width = x_cvstat.scrnwidth * (*x / x_cvstat.scrnwidth);
                if (width < x_cvstat.scrnwidth)
                        width = x_cvstat.scrnwidth;

                height = x_cvstat.scrnheight * (((double)x_cvstat.scrnwidth / x_cvstat.scrnheight) / ((double)x_cvstat.aspect_width / x_cvstat.aspect_height));
                height = x_cvstat.scrnheight * (*y / height);
                if (height < x_cvstat.scrnheight)
                        height = x_cvstat.scrnheight;
        }
        else {
                // Height controls
                height = x_cvstat.scrnheight * (*x / x_cvstat.scrnheight);
                if (height < x_cvstat.scrnheight)
                        height = x_cvstat.scrnheight;

                width = x_cvstat.scrnwidth * (((double)x_cvstat.scrnheight / x_cvstat.scrnwidth) / ((double)x_cvstat.aspect_height / x_cvstat.aspect_width));
                width = x_cvstat.scrnwidth * (*y / width);
                if (width < x_cvstat.scrnwidth)
                        width = x_cvstat.scrnwidth;
        }       

        *x = width;
        *y = height;
}

int main(int argc, char **argv)
{
	int w, h;
	int w2, h2;
	if (argc < 3)
		return 1;

	w = atoi(argv[1]);
	h = atoi(argv[2]);

	w2 = w;
	h2 = h;

	aspect_correct(&w2, &h2);
	printf("%d/%d -> %d/%d\n", w, h, w2, h2);

	w2 = w;
	h2 = h;

	aspect_reverse(&w2, &h2);
	printf("%d/%d -> %d/%d\n", w, h, w2, h2);
}
